@@ -1,18 +1,51 @@
-#!/usr/bin/env php
+#!/usr/local/bin/php-5.3
 <?php
-if (php_sapi_name() != 'cli') {
-    die('Not on the "shell" (php-cli).');
-}
+/*
+ +-----------------------------------------------------------------------+
+ | bin/update.sh                                                         |
+ |                                                                       |
+ | This file is part of the Roundcube Webmail client                     |
+ | Copyright (C) 2010-2011, The Roundcube Dev Team                       |
+ | Licensed under the GNU GPL                                            |
+ |                                                                       |
+ | PURPOSE:                                                              |
+ |   Check local configuration and database schema after upgrading       |
+ |   to a new version                                                    |
+ +-----------------------------------------------------------------------+
+ | Author: Thomas Bruederli <roundcube@gmail.com>                        |
+ +-----------------------------------------------------------------------+
+
+ $Id$
+
+*/
+
 define('INSTALL_PATH', realpath(dirname(__FILE__) . '/..') . '/' );
 
-require_once INSTALL_PATH . 'program/include/iniset.php';
+require_once INSTALL_PATH . 'program/include/clisetup.php';
 require_once INSTALL_PATH . 'installer/rcube_install.php';
 
+// get arguments
+$opts = get_opt(array('v' => 'version'));
+
+// ask user if no version is specified
+if (!$opts['version']) {
+  echo "What version are you upgrading from? Type '?' if you don't know.\n";
+  if (($input = trim(fgets(STDIN))) && preg_match('/^[0-9.]+[a-z-]*$/', $input))
+    $opts['version'] = $input;
+}
+
+if ($opts['version'] && version_compare($opts['version'], RCMAIL_VERSION, '>'))
+  die("Nothing to be done here. Bye!\n");
+
+
 $RCI = rcube_install::get_instance();
 $RCI->load_config();
 
 if ($RCI->configured) {
+  $success = true;
+  
   if ($messages = $RCI->check_config()) {
+    $success = false;
     $err = 0;
 
     // list missing config options
@@ -77,7 +110,7 @@ if ($RCI->configured) {
         // Success!
         if ($write1 && $write2) {
           echo "Done.\n";
-          echo "Your configuration files are now up-tp-date!\n";
+          echo "Your configuration files are now up-to-date!\n";
         }
         else {
           echo "Failed to write config files!\n";
@@ -85,7 +118,7 @@ if ($RCI->configured) {
         }
       }
       else {
-        echo "Please update your config files manually according to the above messages.\n";
+        echo "Please update your config files manually according to the above messages.\n\n";
       }
     }
 
@@ -100,15 +133,47 @@ if ($RCI->configured) {
       echo "Please fix your config files and run this script again!\n";
       echo "See ya.\n";
     }
-
   }
-  else {
-    echo "This instance of RoundCube is up-to-date.\n";
+
+  // check database schema
+  if ($RCI->config['db_dsnw']) {
+    $DB = new rcube_mdb2($RCI->config['db_dsnw'], '', false);
+    $DB->db_connect('w');
+    if ($db_error_msg = $DB->is_error()) {
+      echo "Error connecting to database: $db_error_msg\n";
+      $success = false;
+    }
+    else if ($err = $RCI->db_schema_check($DB, false)) {
+      $updatefile = INSTALL_PATH . 'SQL/' . (isset($RCI->db_map[$DB->db_provider]) ? $RCI->db_map[$DB->db_provider] : $DB->db_provider) . '.update.sql';
+      echo "WARNING: Database schema needs to be updated!\n";
+      echo join("\n", $err) . "\n\n";
+      $success = false;
+      
+      if ($opts['version']) {
+        echo "Do you want to run the update queries to get the schmea fixed? (y/N)\n";
+        $input = trim(fgets(STDIN));
+        if (strtolower($input) == 'y') {
+          $success = $RCI->update_db($DB, $opts['version']);
+        }
+      }
+      
+      if (!$success)
+        echo "Open $updatefile and execute all queries below the comment with the currently installed version number.\n";
+    }
+  }
+  
+  // index contacts for fulltext searching
+  if (version_compare($opts['version'], '0.6', '<')) {
+    system(INSTALL_PATH . 'bin/indexcontacts.sh');
+  }
+  
+  if ($success) {
+    echo "This instance of Roundcube is up-to-date.\n";
     echo "Have fun!\n";
   }
 }
 else {
-  echo "This instance of RoundCube is not yet configured!\n";
+  echo "This instance of Roundcube is not yet configured!\n";
   echo "Open http://url-to-roundcube/installer/ in your browser and follow the instuctions.\n";
 }
 
